@@ -3,11 +3,7 @@
 
 import numpy as np
 import ot
-from .benchmark import (
-    setup_backends,
-    exec_bench,
-    convert_to_html_table
-)
+from .benchmark import setup_backends, exec_bench, convert_to_html_table
 
 
 def setup(n_samples):
@@ -33,10 +29,12 @@ if __name__ == "__main__":
         tested_function=lambda *args: ot.bregman.sinkhorn(*args, reg=1, stopThr=1e-7),
         param_list=param_list,
         n_runs=n_runs,
-        warmup_runs=warmup_runs
+        warmup_runs=warmup_runs,
+    )
+    print(
+        convert_to_html_table(
+            results,
+            param_name="Sample size",
+            main_title=f"Sinkhorn Knopp - Averaged on {n_runs} runs",
+        )
     )
-    print(convert_to_html_table(
-        results,
-        param_name="Sample size",
-        main_title=f"Sinkhorn Knopp - Averaged on {n_runs} runs"
-    ))
