@@ -14,7 +14,7 @@ Linear OT mapping estimation
 
 # sphinx_gallery_thumbnail_number = 2
 
-#%%
+# %%
 import os
 from pathlib import Path
 
@@ -28,25 +28,23 @@ import ot
 
 n = 1000
 d = 2
-sigma = .1
+sigma = 0.1
 
 rng = np.random.RandomState(42)
 
 # source samples
 angles = rng.rand(n, 1) * 2 * np.pi
-xs = np.concatenate((np.sin(angles), np.cos(angles)),
-                    axis=1) + sigma * rng.randn(n, 2)
-xs[:n // 2, 1] += 2
+xs = np.concatenate((np.sin(angles), np.cos(angles)), axis=1) + sigma * rng.randn(n, 2)
+xs[: n // 2, 1] += 2
 
 
 # target samples
 anglet = rng.rand(n, 1) * 2 * np.pi
-xt = np.concatenate((np.sin(anglet), np.cos(anglet)),
-                    axis=1) + sigma * rng.randn(n, 2)
-xt[:n // 2, 1] += 2
+xt = np.concatenate((np.sin(anglet), np.cos(anglet)), axis=1) + sigma * rng.randn(n, 2)
+xt[: n // 2, 1] += 2
 
 
-A = np.array([[1.5, .7], [.7, 1.5]])
+A = np.array([[1.5, 0.7], [0.7, 1.5]])
 b = np.array([[4, 2]])
 xt = xt.dot(A) + b
 
@@ -55,10 +53,10 @@ xt = xt.dot(A) + b
 # ---------
 
 plt.figure(1, (5, 5))
-plt.plot(xs[:, 0], xs[:, 1], '+')
-plt.plot(xt[:, 0], xt[:, 1], 'o')
-plt.legend(('Source', 'Target'))
-plt.title('Source and target distributions')
+plt.plot(xs[:, 0], xs[:, 1], "+")
+plt.plot(xt[:, 0], xt[:, 1], "o")
+plt.legend(("Source", "Target"))
+plt.title("Source and target distributions")
 plt.show()
 
 ##############################################################################
@@ -83,17 +81,17 @@ xstgw = xs.dot(Agw) + bgw
 plt.figure(2, (10, 5))
 plt.clf()
 plt.subplot(1, 2, 1)
-plt.plot(xs[:, 0], xs[:, 1], '+')
-plt.plot(xt[:, 0], xt[:, 1], 'o')
-plt.plot(xst[:, 0], xst[:, 1], '+')
-plt.legend(('Source', 'Target', 'Transp. Monge'), loc=0)
-plt.title('Transported samples with Monge')
+plt.plot(xs[:, 0], xs[:, 1], "+")
+plt.plot(xt[:, 0], xt[:, 1], "o")
+plt.plot(xst[:, 0], xst[:, 1], "+")
+plt.legend(("Source", "Target", "Transp. Monge"), loc=0)
+plt.title("Transported samples with Monge")
 plt.subplot(1, 2, 2)
-plt.plot(xs[:, 0], xs[:, 1], '+')
-plt.plot(xt[:, 0], xt[:, 1], 'o')
-plt.plot(xstgw[:, 0], xstgw[:, 1], '+')
-plt.legend(('Source', 'Target', 'Transp. GW'), loc=0)
-plt.title('Transported samples with Gaussian GW')
+plt.plot(xs[:, 0], xs[:, 1], "+")
+plt.plot(xt[:, 0], xt[:, 1], "o")
+plt.plot(xstgw[:, 0], xstgw[:, 1], "+")
+plt.legend(("Source", "Target", "Transp. GW"), loc=0)
+plt.title("Transported samples with Gaussian GW")
 plt.show()
 
 ##############################################################################
@@ -116,11 +114,11 @@ def minmax(img):
 
 
 # Loading images
-this_file = os.path.realpath('__file__')
-data_path = os.path.join(Path(this_file).parent.parent.parent, 'data')
+this_file = os.path.realpath("__file__")
+data_path = os.path.join(Path(this_file).parent.parent.parent, "data")
 
-I1 = plt.imread(os.path.join(data_path, 'ocean_day.jpg')).astype(np.float64) / 256
-I2 = plt.imread(os.path.join(data_path, 'ocean_sunset.jpg')).astype(np.float64) / 256
+I1 = plt.imread(os.path.join(data_path, "ocean_day.jpg")).astype(np.float64) / 256
+I2 = plt.imread(os.path.join(data_path, "ocean_sunset.jpg")).astype(np.float64) / 256
 
 
 X1 = im2mat(I1)
@@ -164,30 +162,30 @@ plt.figure(3, figsize=(14, 7))
 
 plt.subplot(2, 3, 1)
 plt.imshow(I1)
-plt.axis('off')
-plt.title('Im. 1')
+plt.axis("off")
+plt.title("Im. 1")
 
 plt.subplot(2, 3, 4)
 plt.imshow(I2)
-plt.axis('off')
-plt.title('Im. 2')
+plt.axis("off")
+plt.title("Im. 2")
 
 plt.subplot(2, 3, 2)
 plt.imshow(I1t)
-plt.axis('off')
-plt.title('Monge mapping Im. 1')
+plt.axis("off")
+plt.title("Monge mapping Im. 1")
 
 plt.subplot(2, 3, 5)
 plt.imshow(I2t)
-plt.axis('off')
-plt.title('Inverse Monge mapping Im. 2')
+plt.axis("off")
+plt.title("Inverse Monge mapping Im. 2")
 
 plt.subplot(2, 3, 3)
 plt.imshow(I1tgw)
-plt.axis('off')
-plt.title('Gaussian GW mapping Im. 1')
+plt.axis("off")
+plt.title("Gaussian GW mapping Im. 1")
 
 plt.subplot(2, 3, 6)
 plt.imshow(I2tgw)
-plt.axis('off')
-plt.title('Inverse Gaussian GW mapping Im. 2')
+plt.axis("off")
+plt.title("Inverse Gaussian GW mapping Im. 2")
