@@ -16,7 +16,7 @@ def geometricBar(weights, alldistribT):
     """return the weighted geometric mean of distributions"""
     weights, alldistribT = list_to_array(weights, alldistribT)
     nx = get_backend(weights, alldistribT)
-    assert (len(weights) == alldistribT.shape[1])
+    assert len(weights) == alldistribT.shape[1]
     return nx.exp(nx.dot(nx.log(alldistribT), weights.T))
 
 
@@ -28,14 +28,14 @@ def geometricMean(alldistribT):
 
 
 def projR(gamma, p):
-    """return the KL projection on the row constraints """
+    """return the KL projection on the row constraints"""
     gamma, p = list_to_array(gamma, p)
     nx = get_backend(gamma, p)
     return (gamma.T * p / nx.maximum(nx.sum(gamma, axis=1), 1e-10)).T
 
 
 def projC(gamma, q):
-    """return the KL projection on the column constraints """
+    """return the KL projection on the column constraints"""
     gamma, q = list_to_array(gamma, q)
     nx = get_backend(gamma, q)
     return gamma * q / nx.maximum(nx.sum(gamma, axis=0), 1e-10)
