@@ -1,4 +1,4 @@
-"""Tests for module da on Domain Adaptation """
+"""Tests for module da on Domain Adaptation"""
 
 # Author: Remi Flamary <remi.flamary@unice.fr>
 #
@@ -38,8 +38,8 @@ def test_class_jax_tf():
         ns = 150
         nt = 200
 
-        Xs, ys = make_data_classif('3gauss', ns)
-        Xt, yt = make_data_classif('3gauss2', nt)
+        Xs, ys = make_data_classif("3gauss", ns)
+        Xt, yt = make_data_classif("3gauss2", nt)
 
         Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
@@ -51,14 +51,22 @@ def test_class_jax_tf():
 
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
-@pytest.mark.parametrize("class_to_test", [ot.da.EMDTransport, ot.da.SinkhornTransport, ot.da.SinkhornLpl1Transport,
-                                           ot.da.SinkhornL1l2Transport, ot.da.SinkhornL1l2Transport])
+@pytest.mark.parametrize(
+    "class_to_test",
+    [
+        ot.da.EMDTransport,
+        ot.da.SinkhornTransport,
+        ot.da.SinkhornLpl1Transport,
+        ot.da.SinkhornL1l2Transport,
+        ot.da.SinkhornL1l2Transport,
+    ],
+)
 def test_log_da(nx, class_to_test):
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
@@ -71,14 +79,13 @@ def test_log_da(nx, class_to_test):
 
 @pytest.skip_backend("tf")
 def test_sinkhorn_lpl1_transport_class(nx):
-    """test_sinkhorn_transport
-    """
+    """test_sinkhorn_transport"""
 
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns, random_state=42)
-    Xt, yt = make_data_classif('3gauss2', nt, random_state=43)
+    Xs, ys = make_data_classif("3gauss", ns, random_state=42)
+    Xt, yt = make_data_classif("3gauss2", nt, random_state=43)
     # prepare semi-supervised labels
     yt_semi = np.copy(yt)
     yt_semi[np.arange(0, nt, 2)] = -1
@@ -102,15 +109,17 @@ def test_sinkhorn_lpl1_transport_class(nx):
     mu_s = unif(ns)
     mu_t = unif(nt)
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+    )
 
     # test transform
     transp_Xs = otda.transform(Xs=Xs)
     assert_equal(transp_Xs.shape, Xs.shape)
 
-    Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1, random_state=44)[0])
+    Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1, random_state=44)[0])
     transp_Xs_new = otda.transform(Xs_new)
 
     # check that the oos method is working
@@ -120,7 +129,7 @@ def test_sinkhorn_lpl1_transport_class(nx):
     transp_Xt = otda.inverse_transform(Xt=Xt)
     assert_equal(transp_Xt.shape, Xt.shape)
 
-    Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1, random_state=45)[0])
+    Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1, random_state=45)[0])
     transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
     # check that the oos method is working
@@ -143,35 +152,39 @@ def test_sinkhorn_lpl1_transport_class(nx):
     # test unsupervised vs semi-supervised mode
     otda_unsup = ot.da.SinkhornLpl1Transport()
     otda_unsup.fit(Xs=Xs, ys=ys, Xt=Xt)
-    assert np.all(np.isfinite(nx.to_numpy(otda_unsup.coupling_))), "unsup coupling is finite"
+    assert np.all(
+        np.isfinite(nx.to_numpy(otda_unsup.coupling_))
+    ), "unsup coupling is finite"
     n_unsup = nx.sum(otda_unsup.cost_)
 
     otda_semi = ot.da.SinkhornLpl1Transport()
     otda_semi.fit(Xs=Xs, ys=ys, Xt=Xt, yt=yt_semi)
-    assert np.all(np.isfinite(nx.to_numpy(otda_semi.coupling_))), "semi coupling is finite"
+    assert np.all(
+        np.isfinite(nx.to_numpy(otda_semi.coupling_))
+    ), "semi coupling is finite"
     assert_equal(otda_semi.cost_.shape, ((Xs.shape[0], Xt.shape[0])))
     n_semisup = nx.sum(otda_semi.cost_)
 
     # check that the cost matrix norms are indeed different
-    assert np.allclose(n_unsup, n_semisup, atol=1e-7), "semisupervised mode is not working"
+    assert np.allclose(
+        n_unsup, n_semisup, atol=1e-7
+    ), "semisupervised mode is not working"
 
     # check that the coupling forbids mass transport between labeled source
     # and labeled target samples
-    mass_semi = nx.sum(
-        otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
+    mass_semi = nx.sum(otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
     assert mass_semi == 0, "semisupervised mode not working"
 
 
 @pytest.skip_backend("tf")
 def test_sinkhorn_l1l2_transport_class(nx):
-    """test_sinkhorn_transport
-    """
+    """test_sinkhorn_transport"""
 
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns, random_state=42)
-    Xt, yt = make_data_classif('3gauss2', nt, random_state=43)
+    Xs, ys = make_data_classif("3gauss", ns, random_state=42)
+    Xt, yt = make_data_classif("3gauss2", nt, random_state=43)
     # prepare semi-supervised labels
     yt_semi = np.copy(yt)
     yt_semi[np.arange(0, nt, 2)] = -1
@@ -194,15 +207,17 @@ def test_sinkhorn_l1l2_transport_class(nx):
     mu_s = unif(ns)
     mu_t = unif(nt)
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+    )
 
     # test transform
     transp_Xs = otda.transform(Xs=Xs)
     assert_equal(transp_Xs.shape, Xs.shape)
 
-    Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1)[0])
+    Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1)[0])
     transp_Xs_new = otda.transform(Xs_new)
 
     # check that the oos method is working
@@ -222,7 +237,7 @@ def test_sinkhorn_l1l2_transport_class(nx):
     assert_equal(transp_ys.shape[0], ys.shape[0])
     assert_equal(transp_ys.shape[1], len(np.unique(yt)))
 
-    Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1)[0])
+    Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1)[0])
     transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
     # check that the oos method is working
@@ -243,13 +258,17 @@ def test_sinkhorn_l1l2_transport_class(nx):
     n_semisup = nx.sum(otda_semi.cost_)
 
     # check that the cost matrix norms are indeed different
-    assert np.allclose(n_unsup, n_semisup, atol=1e-7), "semisupervised mode is not working"
+    assert np.allclose(
+        n_unsup, n_semisup, atol=1e-7
+    ), "semisupervised mode is not working"
 
     # check that the coupling forbids mass transport between labeled source
     # and labeled target samples
     mass_semi = nx.sum(otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
     mass_semi = otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max]
-    assert_allclose(nx.to_numpy(mass_semi), np.zeros_like(mass_semi), rtol=1e-9, atol=1e-9)
+    assert_allclose(
+        nx.to_numpy(mass_semi), np.zeros_like(mass_semi), rtol=1e-9, atol=1e-9
+    )
 
     # check everything runs well with log=True
     otda = ot.da.SinkhornL1l2Transport(log=True)
@@ -260,14 +279,13 @@ def test_sinkhorn_l1l2_transport_class(nx):
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
 def test_sinkhorn_transport_class(nx):
-    """test_sinkhorn_transport
-    """
+    """test_sinkhorn_transport"""
 
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
@@ -287,15 +305,17 @@ def test_sinkhorn_transport_class(nx):
     mu_s = unif(ns)
     mu_t = unif(nt)
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+    )
 
     # test transform
     transp_Xs = otda.transform(Xs=Xs)
     assert_equal(transp_Xs.shape, Xs.shape)
 
-    Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1)[0])
+    Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1)[0])
     transp_Xs_new = otda.transform(Xs_new)
 
     # check that the oos method is working
@@ -315,7 +335,7 @@ def test_sinkhorn_transport_class(nx):
     assert_equal(transp_ys.shape[0], ys.shape[0])
     assert_equal(transp_ys.shape[1], len(np.unique(yt)))
 
-    Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1)[0])
+    Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1)[0])
     transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
     # check that the oos method is working
@@ -336,12 +356,13 @@ def test_sinkhorn_transport_class(nx):
     n_semisup = nx.sum(otda_semi.cost_)
 
     # check that the cost matrix norms are indeed different
-    assert np.allclose(n_unsup, n_semisup, atol=1e-7), "semisupervised mode is not working"
+    assert np.allclose(
+        n_unsup, n_semisup, atol=1e-7
+    ), "semisupervised mode is not working"
 
     # check that the coupling forbids mass transport between labeled source
     # and labeled target samples
-    mass_semi = nx.sum(
-        otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
+    mass_semi = nx.sum(otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
     assert mass_semi == 0, "semisupervised mode not working"
 
     # check everything runs well with log=True
@@ -350,37 +371,40 @@ def test_sinkhorn_transport_class(nx):
     assert len(otda.log_.keys()) != 0
 
     # test diffeernt transform and inverse transform
-    otda = ot.da.SinkhornTransport(out_of_sample_map='ferradans')
+    otda = ot.da.SinkhornTransport(out_of_sample_map="ferradans")
     transp_Xs = otda.fit_transform(Xs=Xs, Xt=Xt)
     assert_equal(transp_Xs.shape, Xs.shape)
     transp_Xt = otda.inverse_transform(Xt=Xt)
     assert_equal(transp_Xt.shape, Xt.shape)
 
     # test diffeernt transform
-    otda = ot.da.SinkhornTransport(out_of_sample_map='continuous', method='sinkhorn')
+    otda = ot.da.SinkhornTransport(out_of_sample_map="continuous", method="sinkhorn")
     transp_Xs2 = otda.fit_transform(Xs=Xs, Xt=Xt)
     assert_equal(transp_Xs2.shape, Xs.shape)
     transp_Xt2 = otda.inverse_transform(Xt=Xt)
     assert_equal(transp_Xt2.shape, Xt.shape)
 
-    np.testing.assert_almost_equal(nx.to_numpy(transp_Xs), nx.to_numpy(transp_Xs2), decimal=5)
-    np.testing.assert_almost_equal(nx.to_numpy(transp_Xt), nx.to_numpy(transp_Xt2), decimal=5)
+    np.testing.assert_almost_equal(
+        nx.to_numpy(transp_Xs), nx.to_numpy(transp_Xs2), decimal=5
+    )
+    np.testing.assert_almost_equal(
+        nx.to_numpy(transp_Xt), nx.to_numpy(transp_Xt2), decimal=5
+    )
 
     with pytest.raises(ValueError):
-        otda = ot.da.SinkhornTransport(out_of_sample_map='unknown')
+        otda = ot.da.SinkhornTransport(out_of_sample_map="unknown")
 
 
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
 def test_unbalanced_sinkhorn_transport_class(nx):
-    """test_sinkhorn_transport
-    """
+    """test_sinkhorn_transport"""
 
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
@@ -415,7 +439,7 @@ def test_unbalanced_sinkhorn_transport_class(nx):
         assert_equal(transp_ys.shape[0], ys.shape[0])
         assert_equal(transp_ys.shape[1], len(np.unique(yt)))
 
-        Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1)[0])
+        Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1)[0])
         transp_Xs_new = otda.transform(Xs_new)
 
         # check that the oos method is working
@@ -425,7 +449,7 @@ def test_unbalanced_sinkhorn_transport_class(nx):
         transp_Xt = otda.inverse_transform(Xt=Xt)
         assert_equal(transp_Xt.shape, Xt.shape)
 
-        Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1)[0])
+        Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1)[0])
         transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
         # check that the oos method is working
@@ -448,7 +472,9 @@ def test_unbalanced_sinkhorn_transport_class(nx):
         n_semisup = nx.sum(otda_semi.cost_)
 
         # check that the cost matrix norms are indeed different
-        assert np.allclose(n_unsup, n_semisup, atol=1e-7), "semisupervised mode is not working"
+        assert np.allclose(
+            n_unsup, n_semisup, atol=1e-7
+        ), "semisupervised mode is not working"
 
         # check everything runs well with log=True
         otda = ot.da.SinkhornTransport(log=True)
@@ -460,14 +486,13 @@ def test_unbalanced_sinkhorn_transport_class(nx):
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
 def test_emd_transport_class(nx):
-    """test_sinkhorn_transport
-    """
+    """test_sinkhorn_transport"""
 
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
@@ -488,15 +513,17 @@ def test_emd_transport_class(nx):
     mu_s = unif(ns)
     mu_t = unif(nt)
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+    )
 
     # test transform
     transp_Xs = otda.transform(Xs=Xs)
     assert_equal(transp_Xs.shape, Xs.shape)
 
-    Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1)[0])
+    Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1)[0])
     transp_Xs_new = otda.transform(Xs_new)
 
     # check that the oos method is working
@@ -516,7 +543,7 @@ def test_emd_transport_class(nx):
     assert_equal(transp_ys.shape[0], ys.shape[0])
     assert_equal(transp_ys.shape[1], len(np.unique(yt)))
 
-    Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1)[0])
+    Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1)[0])
     transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
     # check that the oos method is working
@@ -544,17 +571,19 @@ def test_emd_transport_class(nx):
     n_semisup = nx.sum(otda_semi.cost_)
 
     # check that the cost matrix norms are indeed different
-    assert np.allclose(n_unsup, n_semisup, atol=1e-7), "semisupervised mode is not working"
+    assert np.allclose(
+        n_unsup, n_semisup, atol=1e-7
+    ), "semisupervised mode is not working"
 
     # check that the coupling forbids mass transport between labeled source
     # and labeled target samples
-    mass_semi = nx.sum(
-        otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
+    mass_semi = nx.sum(otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max])
     mass_semi = otda_semi.coupling_[otda_semi.cost_ == otda_semi.limit_max]
 
     # we need to use a small tolerance here, otherwise the test breaks
-    assert_allclose(nx.to_numpy(mass_semi), np.zeros(list(mass_semi.shape)),
-                    rtol=1e-2, atol=1e-2)
+    assert_allclose(
+        nx.to_numpy(mass_semi), np.zeros(list(mass_semi.shape)), rtol=1e-2, atol=1e-2
+    )
 
 
 @pytest.skip_backend("jax")
@@ -562,15 +591,14 @@ def test_emd_transport_class(nx):
 @pytest.mark.parametrize("kernel", ["linear", "gaussian"])
 @pytest.mark.parametrize("bias", ["unbiased", "biased"])
 def test_mapping_transport_class(nx, kernel, bias):
-    """test_mapping_transport
-    """
+    """test_mapping_transport"""
 
     ns = 20
     nt = 30
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
-    Xs_new, _ = make_data_classif('3gauss', ns + 1)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
+    Xs_new, _ = make_data_classif("3gauss", ns + 1)
 
     Xs, Xt, Xs_new = nx.from_numpy(Xs, Xt, Xs_new)
 
@@ -592,9 +620,11 @@ def test_mapping_transport_class(nx, kernel, bias):
     mu_s = unif(ns)
     mu_t = unif(nt)
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+    )
 
     # test transform
     transp_Xs = otda.transform(Xs=Xs)
@@ -618,8 +648,8 @@ def test_mapping_transport_class_specific_seed(nx):
     ns = 20
     nt = 30
     rng = np.random.RandomState(39)
-    Xs, ys = make_data_classif('3gauss', ns, random_state=rng)
-    Xt, yt = make_data_classif('3gauss2', nt, random_state=rng)
+    Xs, ys = make_data_classif("3gauss", ns, random_state=rng)
+    Xt, yt = make_data_classif("3gauss2", nt, random_state=rng)
     otda = ot.da.MappingTransport(kernel="gaussian", bias=False)
     otda.fit(Xs=nx.from_numpy(Xs), Xt=nx.from_numpy(Xt))
 
@@ -630,8 +660,8 @@ def test_linear_mapping_class(nx):
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xsb, Xtb = nx.from_numpy(Xs, Xt)
 
@@ -665,8 +695,8 @@ def test_linear_gw_mapping_class(nx):
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xsb, Xtb = nx.from_numpy(Xs, Xt)
 
@@ -690,17 +720,16 @@ def test_linear_gw_mapping_class(nx):
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
 def test_jcpot_transport_class(nx):
-    """test_jcpot_transport
-    """
+    """test_jcpot_transport"""
 
     ns1 = 50
     ns2 = 50
     nt = 50
 
-    Xs1, ys1 = make_data_classif('3gauss', ns1)
-    Xs2, ys2 = make_data_classif('3gauss', ns2)
+    Xs1, ys1 = make_data_classif("3gauss", ns1)
+    Xs2, ys2 = make_data_classif("3gauss", ns2)
 
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs1, ys1, Xs2, ys2, Xt, yt = nx.from_numpy(Xs1, ys1, Xs2, ys2, Xt, yt)
 
@@ -708,7 +737,9 @@ def test_jcpot_transport_class(nx):
     ys = [ys1, ys2]
 
     for log in [True, False]:
-        otda = ot.da.JCPOTTransport(reg_e=1, max_iter=10000, tol=1e-9, verbose=True, log=log)
+        otda = ot.da.JCPOTTransport(
+            reg_e=1, max_iter=10000, tol=1e-9, verbose=True, log=log
+        )
 
         # test its computed
         otda.fit(Xs=Xs, ys=ys, Xt=Xt)
@@ -727,25 +758,29 @@ def test_jcpot_transport_class(nx):
         for i in range(len(Xs)):
             # test margin constraints w.r.t. uniform target weights for each coupling matrix
             assert_allclose(
-                nx.to_numpy(nx.sum(otda.coupling_[i], axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+                nx.to_numpy(nx.sum(otda.coupling_[i], axis=0)),
+                mu_t,
+                rtol=1e-3,
+                atol=1e-3,
+            )
 
             if log:
                 # test margin constraints w.r.t. modified source weights for each source domain
 
                 assert_allclose(
                     nx.to_numpy(
-                        nx.dot(otda.log_['D1'][i], nx.sum(otda.coupling_[i], axis=1))
+                        nx.dot(otda.log_["D1"][i], nx.sum(otda.coupling_[i], axis=1))
                     ),
                     nx.to_numpy(otda.proportions_),
                     rtol=1e-3,
-                    atol=1e-3
+                    atol=1e-3,
                 )
 
         # test transform
         transp_Xs = otda.transform(Xs=Xs)
         [assert_equal(x.shape, y.shape) for x, y in zip(transp_Xs, Xs)]
 
-        Xs_new = nx.from_numpy(make_data_classif('3gauss', ns1 + 1)[0])
+        Xs_new = nx.from_numpy(make_data_classif("3gauss", ns1 + 1)[0])
         transp_Xs_new = otda.transform(Xs_new)
 
         # check that the oos method is working
@@ -764,8 +799,7 @@ def test_jcpot_transport_class(nx):
 
 
 def test_jcpot_barycenter(nx):
-    """test_jcpot_barycenter
-    """
+    """test_jcpot_barycenter"""
 
     ns1 = 50
     ns2 = 50
@@ -773,21 +807,30 @@ def test_jcpot_barycenter(nx):
 
     sigma = 0.1
 
-    ps1 = .2
-    ps2 = .9
-    pt = .4
+    ps1 = 0.2
+    ps2 = 0.9
+    pt = 0.4
 
-    Xs1, ys1 = make_data_classif('2gauss_prop', ns1, nz=sigma, p=ps1)
-    Xs2, ys2 = make_data_classif('2gauss_prop', ns2, nz=sigma, p=ps2)
-    Xt, _ = make_data_classif('2gauss_prop', nt, nz=sigma, p=pt)
+    Xs1, ys1 = make_data_classif("2gauss_prop", ns1, nz=sigma, p=ps1)
+    Xs2, ys2 = make_data_classif("2gauss_prop", ns2, nz=sigma, p=ps2)
+    Xt, _ = make_data_classif("2gauss_prop", nt, nz=sigma, p=pt)
 
     Xs1b, ys1b, Xs2b, ys2b, Xtb = nx.from_numpy(Xs1, ys1, Xs2, ys2, Xt)
 
     Xsb = [Xs1b, Xs2b]
     ysb = [ys1b, ys2b]
 
-    prop = ot.bregman.jcpot_barycenter(Xsb, ysb, Xtb, reg=.5, metric='sqeuclidean',
-                                       numItermax=10000, stopThr=1e-9, verbose=False, log=False)
+    prop = ot.bregman.jcpot_barycenter(
+        Xsb,
+        ysb,
+        Xtb,
+        reg=0.5,
+        metric="sqeuclidean",
+        numItermax=10000,
+        stopThr=1e-9,
+        verbose=False,
+        log=False,
+    )
 
     np.testing.assert_allclose(nx.to_numpy(prop), [1 - pt, pt], rtol=1e-3, atol=1e-3)
 
@@ -796,18 +839,19 @@ def test_jcpot_barycenter(nx):
 @pytest.skip_backend("jax")
 @pytest.skip_backend("tf")
 def test_emd_laplace_class(nx):
-    """test_emd_laplace_transport
-    """
+    """test_emd_laplace_transport"""
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
 
     for log in [True, False]:
-        otda = ot.da.EMDLaplaceTransport(reg_lap=0.01, max_iter=1000, tol=1e-9, verbose=False, log=log)
+        otda = ot.da.EMDLaplaceTransport(
+            reg_lap=0.01, max_iter=1000, tol=1e-9, verbose=False, log=log
+        )
 
         # test its computed
         otda.fit(Xs=Xs, ys=ys, Xt=Xt)
@@ -823,15 +867,17 @@ def test_emd_laplace_class(nx):
         mu_t = unif(nt)
 
         assert_allclose(
-            nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3)
+            nx.to_numpy(nx.sum(otda.coupling_, axis=0)), mu_t, rtol=1e-3, atol=1e-3
+        )
         assert_allclose(
-            nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3)
+            nx.to_numpy(nx.sum(otda.coupling_, axis=1)), mu_s, rtol=1e-3, atol=1e-3
+        )
 
         # test transform
         transp_Xs = otda.transform(Xs=Xs)
         [assert_equal(x.shape, y.shape) for x, y in zip(transp_Xs, Xs)]
 
-        Xs_new = nx.from_numpy(make_data_classif('3gauss', ns + 1)[0])
+        Xs_new = nx.from_numpy(make_data_classif("3gauss", ns + 1)[0])
         transp_Xs_new = otda.transform(Xs_new)
 
         # check that the oos method is working
@@ -841,7 +887,7 @@ def test_emd_laplace_class(nx):
         transp_Xt = otda.inverse_transform(Xt=Xt)
         assert_equal(transp_Xt.shape, Xt.shape)
 
-        Xt_new = nx.from_numpy(make_data_classif('3gauss2', nt + 1)[0])
+        Xt_new = nx.from_numpy(make_data_classif("3gauss2", nt + 1)[0])
         transp_Xt_new = otda.inverse_transform(Xt=Xt_new)
 
         # check that the oos method is working
@@ -865,7 +911,10 @@ def test_emd_laplace_class(nx):
 @pytest.mark.skipif(nocvxpy, reason="No CVXPY available")
 def test_nearest_brenier_potential(nx):
     X = nx.ones((2, 2))
-    for ssnb in [ot.da.NearestBrenierPotential(log=True), ot.da.NearestBrenierPotential(log=False)]:
+    for ssnb in [
+        ot.da.NearestBrenierPotential(log=True),
+        ot.da.NearestBrenierPotential(log=False),
+    ]:
         ssnb.fit(Xs=X, Xt=X)
         G_lu = ssnb.transform(Xs=X)
         # 'new' input isn't new, so should be equal to target
@@ -881,23 +930,37 @@ def test_emd_laplace(nx):
     ns = 50
     nt = 50
 
-    Xs, ys = make_data_classif('3gauss', ns)
-    Xt, yt = make_data_classif('3gauss2', nt)
+    Xs, ys = make_data_classif("3gauss", ns)
+    Xt, yt = make_data_classif("3gauss2", nt)
 
     Xs, ys, Xt, yt = nx.from_numpy(Xs, ys, Xt, yt)
     M = ot.dist(Xs, Xt)
     with pytest.raises(ValueError):
-        ot.da.emd_laplace(ot.unif(ns), ot.unif(nt), Xs, Xt, M, sim_param=['INVALID', 'INPUT', 2])
+        ot.da.emd_laplace(
+            ot.unif(ns), ot.unif(nt), Xs, Xt, M, sim_param=["INVALID", "INPUT", 2]
+        )
     with pytest.raises(ValueError):
-        ot.da.emd_laplace(ot.unif(ns), ot.unif(nt), Xs, Xt, M, sim=['INVALID', 'INPUT', 2])
+        ot.da.emd_laplace(
+            ot.unif(ns), ot.unif(nt), Xs, Xt, M, sim=["INVALID", "INPUT", 2]
+        )
 
     # test all margin constraints with gaussian similarity and disp regularisation
-    coupling = ot.da.emd_laplace(ot.unif(ns, type_as=Xs), ot.unif(nt, type_as=Xs), Xs, Xt, M, sim='gauss', reg='disp')
+    coupling = ot.da.emd_laplace(
+        ot.unif(ns, type_as=Xs),
+        ot.unif(nt, type_as=Xs),
+        Xs,
+        Xt,
+        M,
+        sim="gauss",
+        reg="disp",
+    )
 
     assert_allclose(
-        nx.to_numpy(nx.sum(coupling, axis=0)), unif(nt), rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(coupling, axis=0)), unif(nt), rtol=1e-3, atol=1e-3
+    )
     assert_allclose(
-        nx.to_numpy(nx.sum(coupling, axis=1)), unif(ns), rtol=1e-3, atol=1e-3)
+        nx.to_numpy(nx.sum(coupling, axis=1)), unif(ns), rtol=1e-3, atol=1e-3
+    )
 
 
 @pytest.skip_backend("jax")
@@ -966,10 +1029,10 @@ def test_sinkhorn_lpl1_vectorization(nx):
         indices_labels = []
         classes = nx.unique(labels_a)
         for c in classes:
-            idxc, = nx.where(labels_a == c)
+            (idxc,) = nx.where(labels_a == c)
             indices_labels.append(idxc)
         W = nx.ones(M.shape, type_as=M)
-        for (i, c) in enumerate(classes):
+        for i, c in enumerate(classes):
             majs = nx.sum(transp[indices_labels[i]], axis=0)
             majs = p * ((majs + epsilon) ** (p - 1))
             W[indices_labels[i]] = majs
@@ -979,7 +1042,10 @@ def test_sinkhorn_lpl1_vectorization(nx):
         labels_u, labels_idx = nx.unique(labels_a, return_inverse=True)
         n_labels = labels_u.shape[0]
         unroll_labels_idx = nx.eye(n_labels, type_as=transp)[labels_idx]
-        W = nx.repeat(transp.T[:, :, None], n_labels, axis=2) * unroll_labels_idx[None, :, :]
+        W = (
+            nx.repeat(transp.T[:, :, None], n_labels, axis=2)
+            * unroll_labels_idx[None, :, :]
+        )
         W = nx.sum(W, axis=1)
         W = p * ((W + epsilon) ** (p - 1))
         W = nx.dot(W, unroll_labels_idx.T)
