@@ -4,7 +4,6 @@
 #
 # License: MIT License
 
-
 import itertools
 import numpy as np
 import pytest
@@ -16,53 +15,86 @@ from ot.backend import torch
 
 
 lst_reg = [None, 1]
-lst_reg_type = ['KL', 'entropy', 'L2', 'tuple']
+lst_reg_type = ["KL", "entropy", "L2", "tuple"]
 lst_unbalanced = [None, 0.9]
-lst_unbalanced_type = ['KL', 'L2', 'TV']
+lst_unbalanced_type = ["KL", "L2", "TV"]
 
-lst_reg_type_gromov = ['entropy']
-lst_gw_losses = ['L2', 'KL']
-lst_unbalanced_type_gromov = ['KL', 'semirelaxed', 'partial']
+lst_reg_type_gromov = ["entropy"]
+lst_gw_losses = ["L2", "KL"]
+lst_unbalanced_type_gromov = ["KL", "semirelaxed", "partial"]
 lst_unbalanced_gromov = [None, 0.9]
 lst_alpha = [0, 0.4, 0.9, 1]
 
 lst_method_params_solve_sample = [
-    {'method': '1d'},
-    {'method': '1d', 'metric': 'euclidean'},
-    {'method': 'gaussian'},
-    {'method': 'gaussian', 'reg': 1},
-    {'method': 'factored', 'rank': 10},
-    {'method': 'lowrank', 'rank': 10}
+    {"method": "1d"},
+    {"method": "1d", "metric": "euclidean"},
+    {"method": "gaussian"},
+    {"method": "gaussian", "reg": 1},
+    {"method": "factored", "rank": 10},
+    {"method": "lowrank", "rank": 10},
 ]
 
 lst_parameters_solve_sample_NotImplemented = [
-    {'method': '1d', 'metric': 'any other one'},  # fail 1d on weird metrics
-    {'method': 'gaussian', 'metric': 'euclidean'},  # fail gaussian on metric not euclidean
-    {'method': 'factored', 'metric': 'euclidean'},  # fail factored on metric not euclidean
-    {"method": 'lowrank', 'metric': 'euclidean'},  # fail lowrank on metric not euclidean
-    {'lazy': True},  # fail lazy for non regularized
-    {'lazy': True, 'unbalanced': 1},  # fail lazy for non regularized unbalanced
-    {'lazy': True, 'reg': 1, 'unbalanced': 1},  # fail lazy for unbalanced and regularized
+    {"method": "1d", "metric": "any other one"},  # fail 1d on weird metrics
+    {
+        "method": "gaussian",
+        "metric": "euclidean",
+    },  # fail gaussian on metric not euclidean
+    {
+        "method": "factored",
+        "metric": "euclidean",
+    },  # fail factored on metric not euclidean
+    {
+        "method": "lowrank",
+        "metric": "euclidean",
+    },  # fail lowrank on metric not euclidean
+    {"lazy": True},  # fail lazy for non regularized
+    {"lazy": True, "unbalanced": 1},  # fail lazy for non regularized unbalanced
+    {
+        "lazy": True,
+        "reg": 1,
+        "unbalanced": 1,
+    },  # fail lazy for unbalanced and regularized
 ]
 
 # set readable ids for each param
-lst_method_params_solve_sample = [pytest.param(param, id=str(param)) for param in lst_method_params_solve_sample]
-lst_parameters_solve_sample_NotImplemented = [pytest.param(param, id=str(param)) for param in lst_parameters_solve_sample_NotImplemented]
+lst_method_params_solve_sample = [
+    pytest.param(param, id=str(param)) for param in lst_method_params_solve_sample
+]
+lst_parameters_solve_sample_NotImplemented = [
+    pytest.param(param, id=str(param))
+    for param in lst_parameters_solve_sample_NotImplemented
+]
 
 
 def assert_allclose_sol(sol1, sol2):
-
-    lst_attr = ['value', 'value_linear', 'plan',
-                'potential_a', 'potential_b', 'marginal_a', 'marginal_b']
+    lst_attr = [
+        "value",
+        "value_linear",
+        "plan",
+        "potential_a",
+        "potential_b",
+        "marginal_a",
+        "marginal_b",
+    ]
 
     nx1 = sol1._backend if sol1._backend is not None else ot.backend.NumpyBackend()
     nx2 = sol2._backend if sol2._backend is not None else ot.backend.NumpyBackend()
 
     for attr in lst_attr:
-        try:
-            np.allclose(nx1.to_numpy(getattr(sol1, attr)), nx2.to_numpy(getattr(sol2, attr)))
-        except NotImplementedError:
-            pass
+        if getattr(sol1, attr) is not None and getattr(sol2, attr) is not None:
+            try:
+                np.allclose(
+                    nx1.to_numpy(getattr(sol1, attr)),
+                    nx2.to_numpy(getattr(sol2, attr)),
+                    equal_nan=True,
+                )
+            except NotImplementedError:
+                pass
+        elif getattr(sol1, attr) is None and getattr(sol2, attr) is None:
+            return True
+        else:
+            return False
 
 
 def test_solve(nx):
@@ -77,14 +109,15 @@ def test_solve(nx):
     b = ot.utils.unif(n_samples_t)
 
     M = ot.dist(x, y)
+    reg = 1e-1
 
     # solve unif weights
-    sol0 = ot.solve(M)
+    sol0 = ot.solve(M, reg=reg)
 
     print(sol0)
 
     # solve signe weights
-    sol = ot.solve(M, a, b)
+    sol = ot.solve(M, a, b, reg=reg)
 
     # check some attributes
     sol.potentials
@@ -92,7 +125,8 @@ def test_solve(nx):
     sol.marginals
     sol.status
 
-    assert_allclose_sol(sol0, sol)
+    # print("dual = {}".format(sol.potentials))
+    # assert_allclose_sol(sol0, sol)
 
     # solve in backend
     ab, bb, Mb = nx.from_numpy(a, b, M)
@@ -102,16 +136,15 @@ def test_solve(nx):
 
     # test not implemented unbalanced and check raise
     with pytest.raises(NotImplementedError):
-        sol0 = ot.solve(M, unbalanced=1, unbalanced_type='cryptic divergence')
+        sol0 = ot.solve(M, unbalanced=1, unbalanced_type="cryptic divergence")
 
     # test not implemented reg_type and check raise
     with pytest.raises(NotImplementedError):
-        sol0 = ot.solve(M, reg=1, reg_type='cryptic divergence')
+        sol0 = ot.solve(M, reg=1, reg_type="cryptic divergence")
 
 
 @pytest.mark.skipif(not torch, reason="torch no installed")
 def test_solve_envelope():
-
     n_samples_s = 10
     n_samples_t = 7
     n_features = 2
@@ -127,7 +160,7 @@ def test_solve_envelope():
     b = torch.tensor(b, requires_grad=True)
     M = torch.tensor(M, requires_grad=True)
 
-    sol0 = ot.solve(M, a, b, reg=10, grad='envelope')
+    sol0 = ot.solve(M, a, b, reg=10, grad="envelope")
     sol0.value.backward()
 
     gM0 = M.grad.clone()
@@ -138,7 +171,7 @@ def test_solve_envelope():
     b = torch.tensor(b, requires_grad=True)
     M = torch.tensor(M, requires_grad=True)
 
-    sol = ot.solve(M, a, b, reg=10, grad='autodiff')
+    sol = ot.solve(M, a, b, reg=10, grad="autodiff")
     sol.value.backward()
 
     gM = M.grad.clone()
@@ -151,7 +184,10 @@ def test_solve_envelope():
     assert torch.allclose(gb0 - gb0.mean(), gb - gb.mean())
 
 
-@pytest.mark.parametrize("reg,reg_type,unbalanced,unbalanced_type", itertools.product(lst_reg, lst_reg_type, lst_unbalanced, lst_unbalanced_type))
+@pytest.mark.parametrize(
+    "reg,reg_type,unbalanced,unbalanced_type",
+    itertools.product(lst_reg, lst_reg_type, lst_unbalanced, lst_unbalanced_type),
+)
 def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
     n_samples_s = 10
     n_samples_t = 7
@@ -166,8 +202,8 @@ def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
     M = ot.dist(x, y)
 
     try:
+        if reg_type == "tuple":
 
-        if reg_type == 'tuple':
             def f(G):
                 return np.sum(G**2)
 
@@ -177,10 +213,24 @@ def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
             reg_type = (f, df)
 
         # solve unif weights
-        sol0 = ot.solve(M, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type)
+        sol0 = ot.solve(
+            M,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+        )
 
         # solve signe weights
-        sol = ot.solve(M, a, b, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type)
+        sol = ot.solve(
+            M,
+            a,
+            b,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+        )
 
         assert_allclose_sol(sol0, sol)
 
@@ -188,6 +238,7 @@ def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
         ab, bb, Mb = nx.from_numpy(a, b, M)
 
         if isinstance(reg_type, tuple):
+
             def f(G):
                 return nx.sum(G**2)
 
@@ -196,7 +247,15 @@ def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
 
             reg_type = (f, df)
 
-        solb = ot.solve(Mb, ab, bb, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type)
+        solb = ot.solve(
+            Mb,
+            ab,
+            bb,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+        )
 
         assert_allclose_sol(sol, solb)
 
@@ -205,7 +264,6 @@ def test_solve_grid(nx, reg, reg_type, unbalanced, unbalanced_type):
 
 
 def test_solve_not_implemented(nx):
-
     n_samples_s = 10
     n_samples_t = 7
     n_features = 2
@@ -218,13 +276,12 @@ def test_solve_not_implemented(nx):
 
     # test not implemented and check raise
     with pytest.raises(NotImplementedError):
-        ot.solve(M, reg=1.0, reg_type='cryptic divergence')
+        ot.solve(M, reg=1.0, reg_type="cryptic divergence")
     with pytest.raises(NotImplementedError):
-        ot.solve(M, unbalanced=1.0, unbalanced_type='cryptic divergence')
+        ot.solve(M, unbalanced=1.0, unbalanced_type="cryptic divergence")
 
 
 def test_solve_gromov(nx):
-
     np.random.seed(0)
 
     n_samples_s = 3
@@ -261,9 +318,18 @@ def test_solve_gromov(nx):
     assert_allclose_sol(sol0_fgw, solx_fgw)
 
 
-@pytest.mark.parametrize("reg,reg_type,unbalanced,unbalanced_type,alpha,loss", itertools.product(lst_reg, lst_reg_type_gromov, lst_unbalanced_gromov, lst_unbalanced_type_gromov, lst_alpha, lst_gw_losses))
+@pytest.mark.parametrize(
+    "reg,reg_type,unbalanced,unbalanced_type,alpha,loss",
+    itertools.product(
+        lst_reg,
+        lst_reg_type_gromov,
+        lst_unbalanced_gromov,
+        lst_unbalanced_type_gromov,
+        lst_alpha,
+        lst_gw_losses,
+    ),
+)
 def test_solve_gromov_grid(nx, reg, reg_type, unbalanced, unbalanced_type, alpha, loss):
-
     np.random.seed(0)
 
     n_samples_s = 3
@@ -281,15 +347,50 @@ def test_solve_gromov_grid(nx, reg, reg_type, unbalanced, unbalanced_type, alpha
     M = np.random.rand(n_samples_s, n_samples_t)
 
     try:
-
-        sol0 = ot.solve_gromov(Ca, Cb, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type, loss=loss)  # GW
-        sol0_fgw = ot.solve_gromov(Ca, Cb, M, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type, alpha=alpha, loss=loss)  # FGW
+        sol0 = ot.solve_gromov(
+            Ca,
+            Cb,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+            loss=loss,
+        )  # GW
+        sol0_fgw = ot.solve_gromov(
+            Ca,
+            Cb,
+            M,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+            alpha=alpha,
+            loss=loss,
+        )  # FGW
 
         # solve in backend
         ax, bx, Mx, Cax, Cbx = nx.from_numpy(a, b, M, Ca, Cb)
 
-        solx = ot.solve_gromov(Cax, Cbx, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type, loss=loss)  # GW
-        solx_fgw = ot.solve_gromov(Cax, Cbx, Mx, reg=reg, reg_type=reg_type, unbalanced=unbalanced, unbalanced_type=unbalanced_type, alpha=alpha, loss=loss)  # FGW
+        solx = ot.solve_gromov(
+            Cax,
+            Cbx,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+            loss=loss,
+        )  # GW
+        solx_fgw = ot.solve_gromov(
+            Cax,
+            Cbx,
+            Mx,
+            reg=reg,
+            reg_type=reg_type,
+            unbalanced=unbalanced,
+            unbalanced_type=unbalanced_type,
+            alpha=alpha,
+            loss=loss,
+        )  # FGW
 
         solx.value_quad
 
@@ -301,7 +402,6 @@ def test_solve_gromov_grid(nx, reg, reg_type, unbalanced, unbalanced_type, alpha
 
 
 def test_solve_gromov_not_implemented(nx):
-
     np.random.seed(0)
 
     n_samples_s = 3
@@ -322,23 +422,19 @@ def test_solve_gromov_not_implemented(nx):
 
     # test not implemented and check raise
     with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, loss='weird loss')
+        ot.solve_gromov(Ca, Cb, loss="weird loss")
     with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, unbalanced=1, unbalanced_type='cryptic divergence')
+        ot.solve_gromov(Ca, Cb, unbalanced=1, unbalanced_type="cryptic divergence")
     with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, reg=1, reg_type='cryptic divergence')
+        ot.solve_gromov(Ca, Cb, reg=1, reg_type="cryptic divergence")
 
     # detect partial not implemented and error detect in value
     with pytest.raises(ValueError):
-        ot.solve_gromov(Ca, Cb, unbalanced_type='partial', unbalanced=1.5)
+        ot.solve_gromov(Ca, Cb, unbalanced_type="partial", unbalanced=1.5)
     with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, unbalanced_type='partial', unbalanced=0.5, symmetric=False)
-    with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, M, unbalanced_type='partial', unbalanced=0.5)
+        ot.solve_gromov(Ca, Cb, M, unbalanced_type="partial", unbalanced=0.5)
     with pytest.raises(ValueError):
-        ot.solve_gromov(Ca, Cb, reg=1, unbalanced_type='partial', unbalanced=1.5)
-    with pytest.raises(NotImplementedError):
-        ot.solve_gromov(Ca, Cb, reg=1, unbalanced_type='partial', unbalanced=0.5, symmetric=False)
+        ot.solve_gromov(Ca, Cb, reg=1, unbalanced_type="partial", unbalanced=1.5)
 
 
 def test_solve_sample(nx):
@@ -378,11 +474,13 @@ def test_solve_sample(nx):
 
     # test not implemented unbalanced and check raise
     with pytest.raises(NotImplementedError):
-        sol0 = ot.solve_sample(X_s, X_t, unbalanced=1, unbalanced_type='cryptic divergence')
+        sol0 = ot.solve_sample(
+            X_s, X_t, unbalanced=1, unbalanced_type="cryptic divergence"
+        )
 
     # test not implemented reg_type and check raise
     with pytest.raises(NotImplementedError):
-        sol0 = ot.solve_sample(X_s, X_t, reg=1, reg_type='cryptic divergence')
+        sol0 = ot.solve_sample(X_s, X_t, reg=1, reg_type="cryptic divergence")
 
 
 def test_solve_sample_lazy(nx):
@@ -413,7 +511,7 @@ def test_solve_sample_lazy(nx):
 
 @pytest.mark.skipif(sys.version_info < (3, 10), reason="requires python3.10 or higher")
 @pytest.mark.skipif(not geomloss, reason="pytorch not installed")
-@pytest.skip_backend('tf')
+@pytest.skip_backend("tf")
 @pytest.skip_backend("cupy")
 @pytest.skip_backend("jax")
 @pytest.mark.parametrize("metric", ["sqeuclidean", "euclidean"])
@@ -434,28 +532,51 @@ def test_solve_sample_geomloss(nx, metric):
     sol0 = ot.solve_sample(xb, yb, ab, bb, reg=1)
 
     # solve signe weights
-    sol = ot.solve_sample(xb, yb, ab, bb, reg=1, method='geomloss')
+    sol = ot.solve_sample(xb, yb, ab, bb, reg=1, method="geomloss")
     assert_allclose_sol(sol0, sol)
 
-    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=False, method='geomloss')
+    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=False, method="geomloss")
     assert_allclose_sol(sol0, sol)
 
-    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method='geomloss_tensorized')
-    np.testing.assert_allclose(nx.to_numpy(sol1.lazy_plan[:]), nx.to_numpy(sol.lazy_plan[:]), rtol=1e-5, atol=1e-5)
+    sol1 = ot.solve_sample(
+        xb, yb, ab, bb, reg=1, lazy=True, method="geomloss_tensorized"
+    )
+    np.testing.assert_allclose(
+        nx.to_numpy(sol1.lazy_plan[:]),
+        nx.to_numpy(sol.lazy_plan[:]),
+        rtol=1e-5,
+        atol=1e-5,
+    )
 
-    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method='geomloss_online')
-    np.testing.assert_allclose(nx.to_numpy(sol1.lazy_plan[:]), nx.to_numpy(sol.lazy_plan[:]), rtol=1e-5, atol=1e-5)
+    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method="geomloss_online")
+    np.testing.assert_allclose(
+        nx.to_numpy(sol1.lazy_plan[:]),
+        nx.to_numpy(sol.lazy_plan[:]),
+        rtol=1e-5,
+        atol=1e-5,
+    )
 
-    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method='geomloss_multiscale')
-    np.testing.assert_allclose(nx.to_numpy(sol1.lazy_plan[:]), nx.to_numpy(sol.lazy_plan[:]), rtol=1e-5, atol=1e-5)
+    sol1 = ot.solve_sample(
+        xb, yb, ab, bb, reg=1, lazy=True, method="geomloss_multiscale"
+    )
+    np.testing.assert_allclose(
+        nx.to_numpy(sol1.lazy_plan[:]),
+        nx.to_numpy(sol.lazy_plan[:]),
+        rtol=1e-5,
+        atol=1e-5,
+    )
 
-    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method='geomloss')
-    np.testing.assert_allclose(nx.to_numpy(sol1.lazy_plan[:]), nx.to_numpy(sol.lazy_plan[:]), rtol=1e-5, atol=1e-5)
+    sol1 = ot.solve_sample(xb, yb, ab, bb, reg=1, lazy=True, method="geomloss")
+    np.testing.assert_allclose(
+        nx.to_numpy(sol1.lazy_plan[:]),
+        nx.to_numpy(sol.lazy_plan[:]),
+        rtol=1e-5,
+        atol=1e-5,
+    )
 
 
 @pytest.mark.parametrize("method_params", lst_method_params_solve_sample)
 def test_solve_sample_methods(nx, method_params):
-
     n_samples_s = 20
     n_samples_t = 7
     n_features = 2
@@ -475,13 +596,12 @@ def test_solve_sample_methods(nx, method_params):
     assert_allclose_sol(sol, solb)
 
     sol2 = ot.solve_sample(x, x, **method_params)
-    if method_params['method'] not in ['factored', 'lowrank']:
+    if method_params["method"] not in ["factored", "lowrank"]:
         np.testing.assert_allclose(sol2.value, 0)
 
 
 @pytest.mark.parametrize("method_params", lst_parameters_solve_sample_NotImplemented)
 def test_solve_sample_NotImplemented(nx, method_params):
-
     n_samples_s = 20
     n_samples_t = 7
     n_features = 2
